$ErrorActionPreference = "Stop"
$PSNativeCommandUseErrorActionPreference = $true
Set-StrictMode -Version Latest

if ($PSEdition -eq "Core" -and $IsWindows -ne $true) {
    Write-Host "This script is only supported on Windows"
    Exit 1
}
elseif ([System.Environment]::OSVersion.Version.Major -lt 10) {
    Write-Host "This script is only supported on Windows 10 or later"
    Exit 2
}

$srv = Get-Service -Name "SCardSvr" -ErrorAction Ignore

if ($srv.Status -ne "Running") {
    Write-Host "Smart Card Service is not running" -ForegroundColor RED
}

$edition = "x86"

if ($env:PROCESSOR_ARCHITECTURE -eq "ARM64") {
    $edition = "arm64"
}

$env:FWUPD_MSS = 255
# $env:DRIVER_NAME = "ESTKme-RED"
$fwupd = "$PSScriptRoot\bin\fwupd-$edition.exe"

if (-not (Test-Path $fwupd)) {
    Write-Host "fwupd not found at $fwupd"
    Exit 4
}

function Get-Version {
    (& $fwupd "ver" | Out-String).Trim()
}

function Update-Firmware {
    param (
        [string]$version,
        [string]$blobPath
    )
    & $fwupd "$version-dfu" "$PSScriptRoot\files\$blobPath"
}

function Invoke-Upgrade-Firmware {
    $latest = (Get-Content "$PSScriptRoot\VERSION" | Out-String).Trim()
    while ($true) {
        $version = Get-Version
        Write-Host "Current bootloader version: $version"
        switch -regex ($version) {
            "T001V02" {
                Write-Host "Upgrade bootloader from $version to T001V05"
                Update-Firmware "v1" "$version-T001V05"
            }
            "T001V0[34]" {
                Write-Host "Upgrade bootloader from $version to T001V05"
                Update-Firmware "v1" "T001V03_V04-T001V05"
            }
            "T001V05" {
                Write-Host "Upgrade bootloader from $version to T001V06"
                Update-Firmware "v2" "$version-T001V06"
            }
            "T002V01" {
                Write-Host "Upgrade bootloader from $version to T002V06"
                Update-Firmware "v2" "$version-T002V06"
            }
            "T00[012]V06" {
                Write-Host "Upgrade $version ($latest)"
                Update-Firmware "v2" "$version-$latest"
                return
            }
            default {
                Write-Host "Unknown bootloader version"
                return
            }
        }
        Start-Sleep -Seconds 3
    }
}

Invoke-Upgrade-Firmware
