#!/bin/bash
set -euo pipefail

export FWUPD_MSS=255
# export DRIVER_NAME="ESTKme-RED"
FWUPD="$PWD/bin/fwupd.sh"

if [[ ! -f $FWUPD ]]; then
    echo "fwupd not found at $FWUPD"
    exit 1
fi

function get_version() {
    "$FWUPD" ver
}

function fwupd() {
    "$FWUPD" "$1-dfu" "$PWD/files/$2"
}

function upgrade() {
    local LATEST
    local VERSION
    LATEST="$(cat "$PWD/VERSION")"
    while :; do
        VERSION="$(get_version)"
        echo "Current bootloader version: $VERSION"
        case "$VERSION" in
        T001V02)
            echo "Upgrade bootloader from $VERSION to T001V05"
            fwupd v1 "$VERSION-T001V05"
            ;;
        T001V03 | T001V04)
            echo "Upgrade bootloader from $VERSION to T001V05"
            fwupd v1 "T001V03_V04-T001V05"
            ;;
        T001V05)
            echo "Upgrade bootloader from $VERSION to T001V06"
            fwupd v2 "$VERSION-T001V06"
            ;;
        T002V01)
            echo "Upgrade bootloader from $VERSION to T002V06"
            fwupd v2 "$VERSION-T002V06"
            ;;
        T000V06 | T001V06 | T002V06)
            echo "Upgrade $VERSION ($LATEST)"
            fwupd v2 "$VERSION-$LATEST"
            return
            ;;
        *)
            echo "Unknown bootloader version"
            return
            ;;
        esac
        sleep 3
    done
}

upgrade
